package gmodel

import (
	"context"
	"fusenapi/utils/handlers"

	"gorm.io/gorm"
)

// TODO: 使用model的属性做你想做的
type RelaFsResource struct {
	FsResource
	MetaDataMap map[string]interface{} `json:"meta_data_map" gorm:"-"`
}

func (p *FsResourceModel) FindOneById(ctx context.Context, resourceId string) (resp *FsResource, err error) {
	err = p.db.Table(p.name).WithContext(ctx).Where("resource_id =?", resourceId).Take(&resp).Error
	return resp, err
}

func (p *FsResourceModel) Create(ctx context.Context, req *FsResource) (resp *FsResource, err error) {
	err = p.db.Table(p.name).WithContext(ctx).Create(req).Error
	return req, err
}

func (p *FsResourceModel) Update(ctx context.Context, req *FsResource) (resp *FsResource, err error) {
	err = p.db.Table(p.name).WithContext(ctx).Where("resource_id =?", req.ResourceId).Save(req).Error
	return req, err
}

func (p *FsResourceModel) BuilderCreate(ctx context.Context, rowBuilder *gorm.DB, req *FsResource) (resp *FsResource, err error) {
	err = rowBuilder.WithContext(ctx).Create(req).Error
	return req, err
}

func (p *FsResourceModel) BuilderUpdate(ctx context.Context, rowBuilder *gorm.DB, req *FsResource) (resp *FsResource, err error) {
	err = rowBuilder.WithContext(ctx).Where("resource_id =?", req.ResourceId).Save(req).Error
	return req, err
}

func (m *FsResourceModel) FindOneByQuery(ctx context.Context, rowBuilder *gorm.DB, filterMap map[string]string) (*FsResource, error) {
	var resp FsResource

	if filterMap != nil {
		rowBuilder = rowBuilder.Scopes(handlers.FilterData(filterMap))
	}

	result := rowBuilder.WithContext(ctx).Limit(1).Find(&resp)
	if result.Error != nil {
		return nil, result.Error
	} else {
		return &resp, nil
	}
}

func (m *FsResourceModel) RowSelectBuilder(selectData []string) *gorm.DB {
	var rowBuilder = m.db.Table(m.name)

	if selectData != nil {
		rowBuilder = rowBuilder.Select(selectData)
	} else {
		rowBuilder = rowBuilder.Select("*")
	}
	return rowBuilder
}

func (m *FsResourceModel) BuilderTrans(selectData []string) *gorm.DB {
	var rowBuilder = m.db

	if selectData != nil {
		rowBuilder = rowBuilder.Select(selectData)
	} else {
		rowBuilder = rowBuilder.Select("*")
	}
	return rowBuilder
}

// 事务
func (m *FsResourceModel) Trans(ctx context.Context, fn func(ctx context.Context, connGorm *gorm.DB) error) error {

	return m.db.WithContext(ctx).Transaction(func(tx *gorm.DB) error {
		err := fn(ctx, tx)
		return err
	})

	// tx := m.db.Table(m.name).WithContext(ctx)
	// defer func() {
	// 	if r := recover(); r != nil {
	// 		tx.Rollback()
	// 	}
	// }()

	// if err := tx.Error; err != nil {
	// 	tx.Rollback()
	// 	return err
	// }

	// if err := fn(ctx, tx); err != nil {
	// 	tx.Rollback()
	// 	return err
	// }

	// err := tx.Commit().Error
	// if err != nil {
	// 	tx.Rollback()
	// 	return err
	// }

	// return err
}
func (m *FsResourceModel) FindAllByResourceIds(ctx context.Context, resourceIds []string, fields ...string) (resp []FsResource, err error) {
	if len(resourceIds) == 0 {
		return
	}
	db := m.db.WithContext(ctx).Model(&FsResource{}).Where("resource_id in (?)", resourceIds)
	if len(fields) != 0 {
		db = db.Select(fields[0])
	}
	err = db.Find(&resp).Error
	return resp, err
}
func (m *FsResourceModel) TableName() string {
	return m.name
}
