package gmodel

import (
	"context"
)

func (t *FsTagsModel) FindOne(ctx context.Context, id int64) (resp *FsTags, err error) {
	err = t.db.WithContext(ctx).Model(&FsTags{}).Where("`id` = ? and `status` = ?", id, 1).First(&resp).Error
	return resp, err
}
func (t *FsTagsModel) GetAllByIds(ctx context.Context, ids []int64) (resp []FsTags, err error) {
	if len(ids) == 0 {
		return
	}
	err = t.db.WithContext(ctx).Model(&FsTags{}).Where("`id` in(?) and `status` = ?", ids, 1).Find(&resp).Error
	return resp, err
}
func (t *FsTagsModel) GetAllByIdsWithoutStatus(ctx context.Context, ids []int64) (resp []FsTags, err error) {
	if len(ids) == 0 {
		return
	}
	err = t.db.WithContext(ctx).Model(&FsTags{}).Where("`id` in(?)", ids).Find(&resp).Error
	return resp, err
}
func (t *FsTagsModel) GetAllByLevel(ctx context.Context, level int) (resp []FsTags, err error) {
	err = t.db.Model(&FsTags{}).Where("`level` = ? and `status` = ?", level, 1).Find(&resp).Error
	if err != nil {
		return nil, err
	}
	return
}

type GetAllTagByParamsReq struct {
	Ids                 []int64
	Status              *int64
	OrderBy             string
	LevelPrefixLeftLike string //右模糊
	WithChild           bool   //是否包含子层级
	Category            int64  //分类
	Level               int64
}

func (t *FsTagsModel) GetAllTagByParams(ctx context.Context, req GetAllTagByParamsReq) (resp []FsTags, err error) {
	db := t.db.WithContext(ctx).Model(&FsTags{})
	if len(req.Ids) > 0 {
		db = db.Where("`id` in (?)", req.Ids)
	}
	if req.Status != nil {
		db = db.Where("`status` = ?", *req.Status)
	}
	if req.Category != 0 {
		db = db.Where("`category` = ?", req.Category)
	}
	if req.Level > 0 {
		db = db.Where("`level` = ?", req.Level)
	}
	if req.LevelPrefixLeftLike != "" {
		//查询子集
		if req.WithChild {
			db = db.Where("`level_prefix` like ?", req.LevelPrefixLeftLike+`%`)
		} else {
			db = db.Where("`level_prefix` = ?", req.LevelPrefixLeftLike+`%`)
		}
	}
	switch req.OrderBy {
	case "":
		db = db.Order("`id` DESC")
	default:
		db = db.Order(req.OrderBy)
	}
	err = db.Find(&resp).Error
	return resp, err
}
