package authtest

import (
	"fmt"
	"fusenapi/utils/fstests"

	"testing"

	"github.com/474420502/requests"
	"github.com/tidwall/gjson"
)

func TestAcceptCookieLogic(t *testing.T) {
	var err error
	var resp *requests.Response
	var result gjson.Result

	// 获取 session，并携带 JWT token
	ses := fstests.GetSesssion()

	// 向服务器发送 GET 请求，获取 cookie 信息
	resp, err = ses.Post(fmt.Sprintf("http://%s:%d/api/auth/accept-cookie", cnf.Host, cnf.Port)).TestExecute(gserver)
	if err != nil {
		t.Error(err)
	}

	// 使用 gjson 解析返回的 json 数据
	result = resp.Json() // gjson

	// 检查返回值中的 code 字段是否存在，并且值是否为 200
	code := result.Get("code").Int()
	if code != 200 {
		t.Errorf("Invalid code value: %d", code)
	}

	// 检查返回值中的 msg 字段是否存在，并且值是否为 "success"
	msg := result.Get("msg").String()
	if msg != "success" {
		t.Errorf(`Invalid msg value: "%s"`, msg)
	}

	// 检查返回值中的 data 字段是否存在，并且值是否符合预期
	token := result.Get("data.token").String()
	if len(token) == 0 {
		t.Error("Missing token field")
	}
}
