package logic

import (
	"errors"
	"fusenapi/model/gmodel"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"gorm.io/gorm"
	"time"

	"context"

	"fusenapi/server/collection/internal/svc"
	"fusenapi/server/collection/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type CollectProductLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewCollectProductLogic(ctx context.Context, svcCtx *svc.ServiceContext) *CollectProductLogic {
	return &CollectProductLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *CollectProductLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

func (l *CollectProductLogic) CollectProduct(req *types.CollectProductReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	if !userinfo.IsUser() && !userinfo.IsGuest() {
		return resp.SetStatusWithMessage(basic.CodeUnAuth, "please sign in before collect product")
	}
	//查询产品
	productInfo, err := l.svcCtx.AllModels.FsProduct.FindOne(l.ctx, req.ProductId)
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return resp.SetStatusWithMessage(basic.CodeDbRecordNotFoundErr, "the product is not found")
		}
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "faile to get product info")
	}
	//校验下状态
	/*if *productInfo.Status != 1 {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "the product status is unNormal")
	}*/
	//下架了
	if *productInfo.IsShelf == 0 {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "the product is off shelf")
	}
	if *productInfo.IsDel == 1 {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "the product is deleted")
	}
	//查询收藏
	_, err = l.svcCtx.AllModels.FsProductCollection.FindOne(l.ctx, userinfo.UserId, userinfo.GuestId, req.ProductId)
	if err != nil {
		if !errors.Is(err, gorm.ErrRecordNotFound) {
			logx.Error(err)
			return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to check repeat of collection")
		}
		//创建
		now := time.Now().UTC()
		err = l.svcCtx.AllModels.FsProductCollection.Create(l.ctx, &gmodel.FsProductCollection{
			UserId:           &userinfo.UserId,
			GuestId:          &userinfo.GuestId,
			ProductId:        &req.ProductId,
			TemplateTag:      &req.TemplateTag,
			SelectColorIndex: &req.SelectColorIndex,
			Logo:             &req.Logo,
			Ctime:            &now,
			Utime:            &now,
		})
		if err != nil {
			logx.Error(err)
			return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to collect product")
		}
		return resp.SetStatusWithMessage(basic.CodeOK, "The product has been collected successfully")
	}
	return resp.SetStatusAddMessage(basic.CodeOK, "You have collect this product and don`t need to repeat again")
}

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *CollectProductLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }
