package logic

import (
	"encoding/json"
	"errors"
	"fusenapi/model/gmodel"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"fusenapi/utils/metadata"
	"time"

	"context"

	"fusenapi/server/home-user-auth/internal/svc"
	"fusenapi/server/home-user-auth/internal/types"

	"github.com/zeromicro/go-zero/core/logc"
	"github.com/zeromicro/go-zero/core/logx"
	"gorm.io/gorm"
)

type UserLogoTemplateTagSetLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewUserLogoTemplateTagSetLogic(ctx context.Context, svcCtx *svc.ServiceContext) *UserLogoTemplateTagSetLogic {
	return &UserLogoTemplateTagSetLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *UserLogoTemplateTagSetLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

func (l *UserLogoTemplateTagSetLogic) UserLogoTemplateTagSet(req *types.UserLogoTemplateTagSetReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	// 返回值必须调用Set重新返回, resp可以空指针调用 resp.SetStatus(basic.CodeOK, data)
	// userinfo 传入值时, 一定不为null
	if userinfo.IsOnlooker() {
		// 如果是，返回未授权的错误码
		return resp.SetStatus(basic.CodeUnAuth)
	}
	if req.TemplateTag == "" {
		return resp.SetStatus(basic.CodeApiErr, "logo template tag not null")
	}
	var userId int64
	var guestId int64

	if userinfo.IsGuest() {
		// 如果是，使用游客ID和游客键名格式
		guestId = userinfo.GuestId
	} else {
		// 否则，使用用户ID和用户键名格式
		userId = userinfo.UserId
	}

	var nowTime = time.Now().UTC()
	err := l.svcCtx.MysqlConn.WithContext(l.ctx).Transaction(func(tx *gorm.DB) error {
		var module = "profile"
		var userInfo = &gmodel.FsUserInfo{}
		BuilderDB := tx.Model(&gmodel.FsUserInfo{}).Where("module = ?", module)
		if userId > 0 {
			BuilderDB.Where("user_id=?", userId)
		} else {
			BuilderDB.Where("guest_id=?", guestId)
		}
		userInfoFirstRes := BuilderDB.First(userInfo)
		err := userInfoFirstRes.Error
		if err != nil {
			if !errors.Is(err, gorm.ErrRecordNotFound) {
				logc.Errorf(l.ctx, "userInfo First err：%+v", err)
				return err
			}
		}
		var metadataMapOldUserInfo map[string]interface{}
		if userInfo.Metadata != nil {
			err = json.Unmarshal(*userInfo.Metadata, &metadataMapOldUserInfo)
			if err != nil {
				logc.Errorf(l.ctx, "userInfo Metadata Unmarshal err：%+v", err)
				return err
			}
		}

		var metadataChildUserInfo = make(map[string]interface{}, 1)

		userInfoLogoSelectedData, userInfoLogoSelectedEx := metadataMapOldUserInfo["logo_selected"]
		if userInfoLogoSelectedEx {
			logoSelectedId := int64(userInfoLogoSelectedData.(map[string]interface{})["logo_selected_id"].(float64))
			NewFsUserMaterialModel := gmodel.NewFsUserMaterialModel(l.svcCtx.MysqlConn)
			NewFsUserMaterialModelRow := NewFsUserMaterialModel.RowSelectBuilder(nil).Where("id = ?", logoSelectedId)
			userMaterialInfo, err := NewFsUserMaterialModel.FindOne(l.ctx, NewFsUserMaterialModelRow.Model(&gmodel.FsUserMaterial{}))
			if err != nil {
				logc.Errorf(l.ctx, "FsUserMaterial FindOne err：%+v", err)
				return err
			}

			var metadataMapOldUserMaterial map[string]interface{}
			if userMaterialInfo.Metadata != nil {
				err = json.Unmarshal(*userMaterialInfo.Metadata, &metadataMapOldUserMaterial)
				if err != nil {
					logc.Errorf(l.ctx, "userMaterialInfo Metadata Unmarshal err：%+v", err)
					return err
				}
			}
			userMaterialTemplateTagData, userMaterialTemplateTagEx := metadataMapOldUserMaterial["template_tag"]
			if !userMaterialTemplateTagEx {
				logc.Errorf(l.ctx, "userMaterialInfo Metadata template_tag err：%+v", err)
				return err
			}

			userMaterialTemplateTag := userMaterialTemplateTagData.(map[string]interface{})
			userMaterialTemplateTagIndex := userMaterialTemplateTag[req.TemplateTag]

			if userMaterialTemplateTagIndex == nil {
				err = errors.New("TemplateTag not find")
				logc.Errorf(l.ctx, "TemplateTag not find  err：%+v", err)
				return err
			}

			templateTagSelected := make(map[string]interface{}, 3)
			templateTagSelected["color"] = userMaterialTemplateTagIndex
			templateTagSelected["template_tag"] = req.TemplateTag
			templateTagSelected["selected_index"] = req.TemplateTagColorIndex

			metadataChildUserInfo["logo_selected"] = map[string]interface{}{
				"template_tag_selected": templateTagSelected,
			}
		}

		metadataMapUserInfo, err := metadata.SetMetadata(metadataChildUserInfo, metadataMapOldUserInfo)
		if err != nil {
			logc.Errorf(l.ctx, "userInfo Metadata SetMetadata err：%+v", err)
			return err
		}
		metadataBUserInfo, err := json.Marshal(metadataMapUserInfo)
		if err != nil {
			logc.Errorf(l.ctx, "userInfo Metadata marshal err：%+v", err)
			return err
		}
		userInfo.Metadata = &metadataBUserInfo

		if userInfo.Id == 0 {
			// 新增
			userInfo.Module = &module
			userInfo.Ctime = &nowTime
			userInfo.Utime = &nowTime
			userInfo.UserId = &userId
			userInfo.GuestId = &guestId
			resCreate := tx.Model(&userInfo).Create(&userInfo)
			err = resCreate.Error
		} else {
			// 更新
			userInfo.Utime = &nowTime
			resUpdates := tx.Model(&userInfo).Select("metadata").Where("id = ?", userInfo.Id).Updates(&userInfo)
			err = resUpdates.Error
		}

		if err != nil {
			if err != gorm.ErrRecordNotFound {
				logc.Errorf(l.ctx, "FsUserInfo Updates err：%+v", err)
				return err
			}
		}

		return nil
	})
	if err != nil {
		logx.Error(err)
		return resp.SetStatus(basic.CodeDbSqlErr, "set logo fail")
	}
	return resp.SetStatus(basic.CodeOK)
}

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *UserLogoTemplateTagSetLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }
