package logic

import (
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"

	"context"

	"fusenapi/server/info/internal/svc"
	"fusenapi/server/info/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type UpdateProfileBaseLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewUpdateProfileBaseLogic(ctx context.Context, svcCtx *svc.ServiceContext) *UpdateProfileBaseLogic {
	return &UpdateProfileBaseLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *UpdateProfileBaseLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

func (l *UpdateProfileBaseLogic) UpdateProfileBase(req *types.ProfileRequest, userinfo *auth.UserInfo) (resp *basic.Response) {
	// 返回值必须调用Set重新返回, resp可以空指针调用 resp.SetStatus(basic.CodeOK, data)
	// userinfo 传入值时, 一定不为null
	if !userinfo.IsUser() {
		return resp.SetStatus(basic.CodeUnAuth)
	}

	req.ProfileBase.Email = nil

	err := l.svcCtx.AllModels.FsUserInfo.MergeMetadata(userinfo.UserId, req)
	if err != nil {
		logx.Error(err)                                        // 日志记录错误
		return resp.SetStatus(basic.CodeDbSqlErr, err.Error()) // 返回数据库创建错误
	}

	return resp.SetStatus(basic.CodeOK)
}

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *UpdateProfileBaseLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }
