package logic

import (
	"fusenapi/service/repositories"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"

	"context"

	"fusenapi/server/order/internal/svc"
	"fusenapi/server/order/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type CreatePrePaymentByBalanceLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewCreatePrePaymentByBalanceLogic(ctx context.Context, svcCtx *svc.ServiceContext) *CreatePrePaymentByBalanceLogic {
	return &CreatePrePaymentByBalanceLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *CreatePrePaymentByBalanceLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

func (l *CreatePrePaymentByBalanceLogic) CreatePrePaymentByBalance(req *types.CreatePrePaymentByBalanceReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	// 返回值必须调用Set重新返回, resp可以空指针调用 resp.SetStatus(basic.CodeOK, data)
	// userinfo 传入值时, 一定不为null
	if !userinfo.IsUser() {
		// 如果是，返回未授权的错误码
		return resp.SetStatus(basic.CodeUnAuth)
	}

	res, err := l.svcCtx.Repositories.NewOrder.CreatePrePaymentByBalance(l.ctx, &repositories.CreatePrePaymentByBalanceReq{
		UserId:    userinfo.UserId,
		OrderSn:   req.OrderSn,
		Country:   "US",
		Currency:  "usd",
		StripeKey: l.svcCtx.Config.PayConfig.Stripe.Key,
	})

	if err != nil {
		return resp.SetStatus(&res.ErrorCode)
	}

	return resp.SetStatus(basic.CodeOK, map[string]interface{}{
		"order_detail": res.OrderDetail,
		"order_pay":    res.OrderPay,
	})
}

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *CreatePrePaymentByBalanceLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }
