package logic

import (
	"fusenapi/service/repositories"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"

	"context"

	"fusenapi/server/order/internal/svc"
	"fusenapi/server/order/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type OrderListLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewOrderListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *OrderListLogic {
	return &OrderListLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *OrderListLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

func (l *OrderListLogic) OrderList(req *types.OrderListReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	// 返回值必须调用Set重新返回, resp可以空指针调用 resp.SetStatus(basic.CodeOK, data)
	// userinfo 传入值时, 一定不为null
	if !userinfo.IsUser() {
		// 如果是，返回未授权的错误码
		return resp.SetStatus(basic.CodeUnAuth)
	}
	res, err := l.svcCtx.Repositories.NewOrder.List(l.ctx, &repositories.ListReq{
		UserId:         userinfo.UserId,
		DeliveryMethod: req.DeliveryMethod,
		OrderCycle:     req.OrderCycle,
		CurrentPage:    req.CurrentPage,
		PerPage:        req.PerPage,
	})
	if err != nil {
		return resp.SetStatus(basic.CodeApiErr)
	}

	return resp.SetStatus(basic.CodeOK, map[string]interface{}{
		"list": res.OrderDetailList,
		"meta": res.Meta,
	})
}

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *OrderListLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }
