package logic

import (
	"fusenapi/model/gmodel"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"

	"context"

	"fusenapi/server/upload/internal/svc"
	"fusenapi/server/upload/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
	"gorm.io/gorm"
)

type UploadCallbackLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewUploadCallbackLogic(ctx context.Context, svcCtx *svc.ServiceContext) *UploadCallbackLogic {
	return &UploadCallbackLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *UploadCallbackLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *UploadCallbackLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }

func (l *UploadCallbackLogic) UploadCallback(req *types.UploadCallbackReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	// 返回值必须调用Set重新返回, resp可以空指针调用 resp.SetStatus(basic.CodeOK, data)
	// userinfo 传入值时, 一定不为null
	var metadataB []byte
	if req.Metadata != "" {
		metadataB = []byte(req.Metadata)
	}
	var userId int64
	var guestId int64

	// 检查用户是否是游客
	if userinfo.IsGuest() {
		// 如果是，使用游客ID和游客键名格式
		guestId = userinfo.GuestId
	} else {
		// 否则，使用用户ID和用户键名格式
		userId = userinfo.UserId
	}

	// 定义存储桶名称
	var bucketName *string

	// 根据类别选择存储桶
	switch req.UploadBucket {
	case 2:
		bucketName = basic.TempfileBucketName
	default:
		bucketName = basic.StorageBucketName
	}

	ctx := l.ctx
	err := l.svcCtx.MysqlConn.Transaction(func(connGorm *gorm.DB) error {
		resourceModelTS := gmodel.NewFsResourceModel(connGorm)
		transBuilder := resourceModelTS.BuilderTrans(nil)
		transBuilderFind := transBuilder.Where("resource_id =?", req.ResourceId)
		resourceInfo, err := resourceModelTS.FindOneByQuery(ctx, transBuilderFind, nil)
		if err != nil {
			return err
		}

		var version string = "0.0.1"
		var fsResource = &gmodel.FsResource{}
		fsResource.UserId = &userId
		fsResource.GuestId = &guestId
		fsResource.ResourceId = req.ResourceId
		fsResource.ResourceType = &req.ResourceType
		fsResource.ResourceUrl = &req.ResourceUrl
		fsResource.Metadata = &metadataB
		fsResource.ApiType = &req.ApiType
		fsResource.Source = &req.Source
		fsResource.BucketName = bucketName
		fsResource.Version = &version

		if resourceInfo.ResourceId == "" {
			_, err = resourceModelTS.BuilderCreate(ctx, transBuilder, fsResource)
		} else {
			_, err = resourceModelTS.BuilderUpdate(ctx, transBuilder, fsResource)
		}
		return err
	})
	if err != nil {
		logx.Error(err)
		return resp.SetStatus(basic.CodeFileUploadErr, "file upload err,UploadCallback failed")
	}
	return resp.SetStatus(basic.CodeOK)
}
