package main

import (
	"context"
	"flag"
	"fmt"
	"fusenapi/server/websocket/internal/logic"
	"net/http"

	"fusenapi/utils/auth"
	"fusenapi/utils/fsconfig"

	"fusenapi/server/websocket/internal/config"
	"fusenapi/server/websocket/internal/handler"
	"fusenapi/server/websocket/internal/svc"

	"github.com/zeromicro/go-zero/rest"
)

var configFile = flag.String("f", "etc/websocket.yaml", "the config file")

func main() {
	flag.Parse()

	var c config.Config
	fsconfig.StartNacosConfig(*configFile, &c, nil)

	server := rest.MustNewServer(c.RestConf, rest.WithCustomCors(auth.FsCors, func(w http.ResponseWriter) {
	}))
	defer server.Stop()

	ctx := svc.NewServiceContext(c)
	handler.RegisterHandlers(server, ctx)
	ctx1 := context.Background()
	ctx1, cancel := context.WithCancel(ctx1)
	defer cancel()
	//消费公共通知队列的数据
	go logic.ConsumeCommonCacheData(ctx1)
	//消费用户索引创建/删除/发送消息中的任务数据
	go logic.ConsumeUserConnPoolCtlChanData(ctx1)
	//消费连接统计信息
	go logic.ConsumeWebsocketConnectCountCtlChanData(ctx1)
	fmt.Printf("Starting server at %s:%d...\n", c.Host, c.Port)
	server.Start()
}
